#!/bin/sh

call_preupgrade(){
    [ $TEST -eq 1 ] && return 0
    local IMAGE BACKUP MTD MEMORY VERSION_IMAGE VERSION_CURRENT idx
    critical_error() {
        local led
        local message="$1"
        local leds=$(ls /sys/class/leds/)
        for led in $leds
        do
            echo timer > /sys/class/leds/$led/trigger
        done

        mkdir /www/api/info.d/

        cat <<EOF > /www/api/info.d/uboot.fail
#!/bin/sh
echo '["ERROR", ["UBoot Update", "Failed"], ["Exit Code", "-124"], ["Recommendation", "ALARM!!!!"]]]}'
(kill -9 \$(ps ww | awk '/\/www\/current.info/{print \$1}'))&>/dev/null&   
EOF

        chmod +x /www/api/info.d/uboot.fail

        _do_baduga(){
            while [ 1 ]; do
                logger -p1 "U-BOOT UPDATE ERROR. DO NOT REBOOT THE DEVICE"
                logger -p1 "Recommendation: ALARM!!!!"
                if [ -n "$message" ]; then
                    logger -p1 "$message"
                fi
                sleep 5
            done
        }
        [ "$INCLUDE_PREUPGRADE" == "1" ] || _do_baduga
        _do_baduga&
        return 1
    }

    update() {
        local UBOOT=$1
        mtd write $UBOOT u-boot
        if [ $? = 0 ]; then
            mtd verify $UBOOT u-boot
            if [ $? = 0 ]; then
                echo "U-Boot: Updated" >> /opt/log-uboot
                return 0
            fi
        fi
    }

    MTD="/dev/mtd0"
    MEMORY=$(cat /proc/meminfo | awk -F' ' '/MemTotal/{print $2}')

    if [ "$MEMORY" -gt 100000 ]; then
        MEMORY="128Mb"
    else
        MEMORY="64Mb"
    fi

    IMAGE=$(find /tmp/preupgrade/sysupgrade-*/uboot/* | grep "$MEMORY")
    BACKUP="/tmp/temp_mtd"

    if [ -z "$IMAGE" ]; then
        critical_error "Not found U-Boot image for memory size $MEMORY"
    fi

    dd if=$MTD of=$BACKUP 2>/dev/null
    if [ $? != "0" ]; then
        critical_error "Error reading with $MTD"
    fi

    VERSION_IMAGE=$(strings $IMAGE | awk -F' ' '/U-Boot build:/{print $3}')
    VERSION_CURRENT=$(strings $BACKUP | awk -F' ' '/U-Boot build:/{print $3}')

    if [ -z "$VERSION_CURRENT" ] || [ "$VERSION_CURRENT" -lt "$VERSION_IMAGE" ]; then
        idx=0
        while [ $idx -le 5 ]; do
            update $IMAGE && return 0
            sleep 2
            idx=$(expr $idx + 1)
        done

        idx=0
        while [ $idx -le 5 ]; do
            update $BACKUP && return 0
            sleep 2
            idx=$(expr $idx + 1)
        done

        critical_error
    fi
}

[ "$INCLUDE_PREUPGRADE" == "1" ] || call_preupgrade